#!/usr/bin/perl -w 

# Cliffw OSDL 1/2004
# Detect the operating system version
# Detect the version of disk tools required
# Build the tool if needed
# Create disk partitions either from command line or from
# a config file
#

package STP::Client::DiskHandler;

# KEES: consider making this an object, rather than a list of functions.
#       that way you can have more configurable behavior (for, say, the
#       $DEBUG value).  Just drop the exporter stuff and add a "new".

use strict;
use English;
use FileHandle;
use vars qw(@ISA @EXPORT);
use Exporter;
use Cwd;
use STP::Client::Tools;
my $DEBUG = 0;

@ISA = qw(Exporter);

@EXPORT = qw(
  create_disk_layout
  assign_mountinfo
  disk_tool_check
  make_fs
  clean_spindle
  setup_tools
  assign_single
  make_one_disk
);

# Alarm functions
$SIG{ ALRM } = \&time_out;

sub time_out {
    print "Time in function exceeded\n";
    exit -1;
}

#
# Current versions
# We declare the current versions of everything as globals.
# All the bits should be available from a local directory or a wget host
my $wget_host = "http://stp/data/stp-tools/";

# RPMs
my $suse_xfs_rpm = "popt-devel-1.7-70.i586.rpm";
my $rh9_xfs_rpm  = "xfsprogs-2.5.6-1.i386.rpm";

# Tarballs - we list both version name and compressed file name
# XFS
my $xfs_ver  = "xfsprogs-2.0.3";
my $xfs_file = "xfsprogs-2.0.3.src.tar.gz";

# JFS
my $jfs_ver  = "jfsutils-1.0.20";
my $jfs_file = "jfsutils-1.0.20.tar.gz";

# e2fsprogs
my $e2fs_ver  = "e2fsprogs-1.27";
my $e2fs_file = "e2fsprogs-1.27.tar.gz";

=head1 NAME

DiskHandler

=head1 SYNOPSIS

Creates disk partitions for STP tests. Installs all filesystem tools
needed. Supports ext2,ext3,jfs,xfs,reiserfs(v3), raw
Driven by text mount information file.

=head1 Input Files

Use '|' as a separator
Fields:
Label |
File system type |
Minimum size (KB) |
[ mkfs command ] |
[ options for 'mount' ] |
Mount point 

=head1 NOTES

Minimum size is used when assigning partitions - disks are assigned from
smallest -> largest. Size is a minimum, the physical disk must be greater
than or equal to the disk size. So an arbitrary size can be used to sort 
the disk layout. The disk routine assumes the entire spindle will be made
into one partition. The mkfs command and mount options are optional, and should
be replaced with space if unused

=head1 EXAMPLE

Test_1|ext3|200| | |/opt
Test_2|ext3|100| mke2fs -Fj| -t ext3 -o data=writeback | /mnt/testdata
Test_3|reiserfs|50| | | /mnt/testdata2

=cut

=head1 FUNCTION disk_tool_check
# Detect the version of disk tools required
# This section will require lot's of tweaking, since
# we rely on config management and other stuff
# We should know first what type of disk partition we want
# and what our operation system is.
=cut

sub disk_tool_check {
    my ( $fs_type ) = @_;
    my $os_version = os_detect();

    # A series of if's
    if ( $os_version eq 'Debian' ) {

        if ( $fs_type eq 'ext2' )     { return 0; }
        if ( $fs_type eq 'ext3' )     { return 0; }
        if ( $fs_type eq 'reiserfs' ) { return 0; }
        if ( $fs_type eq 'raw' )      { return 0; }
        if ( $fs_type eq 'xfs' )      {
            install_deb_utils();
            return install_xfs();
        }
        if ( $fs_type eq 'jfs' ) {
            install_deb_utils();
            return install_jfs();
        }
        return 0;
    } elsif ( $os_version eq 'SuSE' ) {
        if ( $fs_type eq 'ext2' )     { return 0; }
        if ( $fs_type eq 'ext3' )     { return 0; }
        if ( $fs_type eq 'reiserfs' ) { return 0; }
        if ( $fs_type eq 'raw' )      { return 0; }
        if ( $fs_type eq 'xfs' )      {
            return install_rpm( $suse_xfs_rpm, $wget_host );
        }
        if ( $fs_type eq 'jfs' ) {
            return install_jfs();
        }
        return 0;
    } elsif ( $os_version eq 'RH_9' ) {
        if ( $fs_type eq 'ext2' )     { return 0; }
        if ( $fs_type eq 'ext3' )     { return 0; }
        if ( $fs_type eq 'reiserfs' ) { return 0; }
        if ( $fs_type eq 'raw' )      { return 0; }
        if ( $fs_type eq 'xfs' )      {
            return 0;
        }
        if ( $fs_type eq 'jfs' ) {
            return 0;
        }
        return 0;
    } elsif ( $os_version eq 'RH_7.3' ) {
        if ( $fs_type eq 'ext2' )     { return 0; }
        if ( $fs_type eq 'ext3' )     { return 0; }
        if ( $fs_type eq 'reiserfs' ) { return 0; }
        if ( $fs_type eq 'raw' )      { return 0; }
        if ( $fs_type eq 'xfs' )      {
            install_e2fsprogs();
            return install_xfs_tar();
        }
        if ( $fs_type eq 'jfs' ) {
            install_e2fsprogs();
            return install_jfs();
        }
        return 0;
    } elsif ( $os_version eq 'RH_7.1' ) {
        if ( $fs_type eq 'ext2' )     { return 0; }
        if ( $fs_type eq 'ext3' )     { return 0; }
        if ( $fs_type eq 'reiserfs' ) { return 0; }
        if ( $fs_type eq 'raw' )      { return 0; }
        if ( $fs_type eq 'xfs' )      {
            install_e2fsprogs();
            return install_xfs_tar();
        }
        if ( $fs_type eq 'jfs' ) {
            install_e2fsprogs();
            return install_jfs();
        }
        return 0;
    } else {
        print STDERR "Can't figure out distro type\n";
        return -1;
    }

}

# Parse the mount info file, make sure we have all the
# different types of filesystem tools
# We use a named file in /tmp to record the test
# to avoid repeated installs and modprobes for
# both the list and command line version
# Also, the leaves a nice record of success
sub setup_tools {
    my ( $mntlist ) = @_;
    my $indx;
    my $res = 0;

    foreach my $fstype ( map { $_->{ 'fstype' } } @{ $mntlist } ) {
        my $touchfile = "/tmp/" . $fstype . ".isokay";
        if ( !-f $touchfile ) {
            if ( $fstype ne 'ext2' ) {
                $res = system( "modprobe $fstype" );
            } else {
                $res = 0;
            }    # ext2 doesn't need modprobe
            if ( $res != 0 ) { 
		print STDERR "modprobe issue\n";
		return -1; 
			}
            $res = disk_tool_check( $fstype );
            if ( $res != 0 ) { 
		print STDERR "disk_tool_check failed\n";
		return -1; }
            `touch $touchfile`;
        }
    }
    return $res;

}

# Build the tool if needed
sub install_deb_utils {

    # KEES: again, use a full path, check return code
    #       I generally write a function named "System" that handles all
    #       this, spitting out the command line that failed, etc.
    #
    # Fakeout for my testing
    `apt-get install uuid-dev`;
    return 0;
}

sub install_e2fsprogs {
    my @build_commands =
      ( "./configure", "make", "make install", "make install-libs" );
    my $result;

    $result = get_bits( $e2fs_file, $wget_host );
    if ( $result == 0 ) {
        $result = install_tar( $e2fs_ver, $e2fs_file, @build_commands );
    } else {
        print STDERR "Failed to get bits\n";
    }
    return $result;
}

sub install_xfs_tar {
    my @build_commands = ( "make configure", "make", "make install" );
    my $result;

    $result = get_bits( $xfs_file, $wget_host );
    if ( $result == 0 ) {
        $result = install_tar( $xfs_ver, $xfs_file, @build_commands );
    } else {
        print STDERR "Failed to get bits\n";
    }
    return $result;
}

sub install_jfs {
    my @build_commands = ( "./configure", "make", "make install" );
    my $result;

    $result = get_bits( $jfs_file, $wget_host );
    if ( $result == 0 ) {
        $result = install_tar( $jfs_ver, $jfs_file, @build_commands );
    } else {
        print STDERR "Failed to get bits\n";
    }
    return $result;
}

# create hash of disk names and sizes from /proc/partitions
sub get_disk_proc {
    my $disk_list;

    # Hardcoded name here
    open( F, "</proc/partitions" ) || return undef;

    while ( <F> ) {
        chomp;
        my ( $dummy, $major, $minor, $blocks, $name ) = split /\s+/;
        unless ( $name ) { next; }
        if ( $major eq "minor" ) { next; }
        if ( $name =~ m/\d+/ ) { next; }

        #skip .da
        if ( $name =~ m/.da$/ ) { next; }
        if ( $DEBUG ) { print "Disk $name size $blocks\n"; }
        $disk_list->{ $name } = $blocks;
    }
    close F;
    return $disk_list;
}

sub get_disk_config {
    my ( $disk_file ) = @_;
    my $disk_list;

    # Hardcoded name here
    open( F, "<$disk_file" ) || return -1;

    # read the disk.info file
    while ( my $dl = <F> ) {
        chomp( $dl );
        if ( ( $dl =~ /^#/ ) || ( $dl =~ /^$/ ) ) { next; }
        my ( $dev, $fakemnt, $size, $raid, $spindle, $nulls ) = split /,/, $dl;
        $dev =~ s/\/dev\///;
        $size = $size * 1024;
        if ( $DEBUG ) { print "Found $dev size $size\n"; }
        $disk_list->{ $dev } = $size;
    }
    close F;
    return $disk_list;
}

# Read a user supplied file of options, separated by '|'
# Options:
# Label
# File system type
# preferred min size
# Command to create filesystem [optional]
# options for mount command [optional]
# Mount point
# End options
# We could have multiple lines in this file with the same text, so we will
# add an Index
# In reality, size and name are also optional. All else will return undef
sub get_mountinfo {
    my ( $mntfile ) = @_;
    my $mntinfo = [];

    open( F, "<$mntfile" ) || return undef;
    while ( my $ln = <F> ) {
        chomp( $ln );
        my $mnt_list;
        if ( ( $ln =~ /^#/ ) || ( $ln =~ /^$/ ) ) { next; }
        my ( $mhname, $mfstype, $msize, $mFmtCmd, $mMntOpts, $mMnt ) =
          split '\|', $ln;
        $mnt_list->{ "name" } = $mhname;
        unless ( $mfstype ) { return undef; }
        $mnt_list->{ "fstype" } = $mfstype;
        $mnt_list->{ "size" }   = $msize;

        if ( ( !$mFmtCmd ) || ( $mFmtCmd eq " " ) ) {
            $mnt_list->{ "FmtCmd" } = "NULL";
        } else {
            $mnt_list->{ "FmtCmd" } = $mFmtCmd;
        }
        if ( ( !$mMntOpts ) || ( $mMntOpts eq " " ) ) {
            $mnt_list->{ "MntOpts" } = "NULL";
        } else {
            $mnt_list->{ "MntOpts" } = $mMntOpts;
        }
        unless ( $mMnt ) { return undef; }
        $mnt_list->{ "Mnt" } = $mMnt;
        push( @$mntinfo, $mnt_list );
    }
    close( F );
    return $mntinfo;
}

# Return a string that can be used as a hash key or directly as a name
# Input is a reference to a hash of disk sizes
sub get_largest_disk {
    my ( $list ) = @_;

    # KEES: Here's an unholy way to shrink this whole function down:
    return ( sort { $list->{ $b } <=> $list->{ $a } } keys %$list )[ 0 ];

}

# Return largest request in mountinfo file.
# Name of mountinfo file is required for parameter
# returns index value for mountinfo array
sub get_largest_request {
    my ( $list )   = @_;
    my $size       = 0;
    my $disk_index = -1;
    my $indx;

    # KEES:
    # return (sort { $list->[$b]->{'size'} <=> $list->[$a]->{'size'} }
    # @$list)[0];
    for ( $indx = 0; $indx <= $#{ $list }; $indx++ ) {
        print STDERR
"size compare: $list->[ $indx ]->{ 'size' } funk $size index $indx $list\n";
        if ( $list->[ $indx ]->{ "size" } > $size ) {
            $size       = $list->[ $indx ]->{ "size" };
            $disk_index = $indx;
        }
    }
    return $disk_index;
}

=head1 FUNCTION
assign_single( mount_info ref )

Used for the command line wrapper. Given a request, find the larges unmounted disk
that will satisfy it. For code re-use, input is a reference to a list. List has
single hash item. Done this way so i don't have to re-write make_fs 

=cut

sub assign_single {
    my ( $mntinfo ) = @_;
    my $flag        = 1;
    my $disklist    = get_disk_proc();
    my $rescode;
    my $min_size;

    unless ( defined( $disklist ) ) {
        print STDERR "Failed to get disklist from /proc\n";
        return -1;
    }

    $min_size = $mntinfo->[ 0 ]->{ 'size' };
    # Search for the largest unmounted disk
    while ( $flag ) {
        my $big_disk = get_largest_disk( $disklist );
        unless ( defined( $big_disk ) ) {
            print STDERR "No more disks available\n";
            return -1;
        }
        my $bg_size = $disklist->{ $big_disk };
        if ( $bg_size == 0 ) {
            print STDERR "No more disks available\n";
            return -1;
        }

        # check if mounted
        my @cmd = ( "grep", $big_disk, "/proc/mounts" );
        $rescode = system( @cmd );
        if ( $rescode == 0 ) {
            $disklist->{ $big_disk } = 0;

            # Try again
            # otherwise, check size
        } elsif ( $bg_size < $min_size ) {
            print STDERR "No disk big enough available\n";
            return -1;
        } else {
            $flag = 0;                                     # exit loop
            $mntinfo->[ 0 ]->{ 'physical' } = $big_disk;
        }

    }
    return $mntinfo;
}

# I am still not sure how to to the matchup between desired disks and the config
# For now, we will match the largest disk with the largest request
# Repeat until all mount requests are consumed
# KEES: This "greedy" approach will actually tend to be the most correct.
#       Stick with it until you get a bad case, I guess.
sub assign_mountinfo {
    my ( $mntlist, $mntname ) = @_;
    my $outlist  = get_mountinfo( $mntname );
    my $disklist = get_disk_proc();
    my $indx;
    my $mntsize  = $#{ $mntlist };              # array starts from 0
    my $disksize = scalar( keys %$disklist );
    $mntsize++;

    if ( $mntsize > $disksize ) {
        print STDERR "$mntsize disks requested $disksize disks found - ERROR\n";
        return -1;
    }

    unless ( ( defined( $outlist ) ) && ( defined( $disklist ) ) ) {
        print STDERR
"Failed to get inputs to assign_mountinfo mntlist $mntlist disklist $disklist\n";
        return -1;
    }

    for ( $indx = 0; $indx <= $#{ $mntlist }; $indx++ ) {
        my $biggest  = get_largest_request( $mntlist );
        my $big_disk = get_largest_disk( $disklist );

        # KEES: and here you can test for "undef" instead of eq "-1"
        if ( ( $biggest == -1 ) || ( !defined( $big_disk ) ) ) {
            print STDERR
              "get_request failed biggest $biggest big-disk $big_disk\n";
            return -1;
        }

        my $big_size = $mntlist->[ $biggest ]->{ 'size' };
        my $bd_size  = $disklist->{ $big_disk };
        unless ( $bd_size >= $big_size ) {
            print STDERR "size compare failed\n";
            return -1;
        }

        # Otherwise, assign the largest, and go on
        $outlist->[ $biggest ]->{ 'physical' } = $big_disk;

        # KEES: maybe you want to "delete" the entries instead?
        # delete $mntlist->[ $biggest ];
        # delete $disklist->{ $big_disk };
        $disklist->{ $big_disk } = 0;
        $mntlist->[ $biggest ]->{ 'size' } = 0;
    }
    return $outlist;

}

# Does the umount and fsck on a specific spindle.
sub clean_spindle {
    my ( $dev ) = @_;
    my $dev_string = "/dev/" . $dev;

    my $ONE     = '1';
    my $umt     = 'umount ' . $dev_string . $ONE;
    my $rescode = system $umt;
    alarm( 3000 );

    open( FDISK, "| sfdisk $dev_string " ) or return -1;
    my $res  = print FDISK "0,,83\n";
    my $resc = close( FDISK );

    alarm( 0 );
    unless ( ( $res ) && ( $resc ) ) { return -1; }
    return 0;
}

# This function takes our list of requested mounts with the list of physical
# disk - if a mount command and/or mount options exist we use, otherwise
# we do a default for that fstype
sub make_fs {
    my ( $mountlist ) = @_;
    my $indx;
    my $rescode;
    my $realres;

    # Goofy fakeout
    my $ONE = '1';

    for ( $indx = 0; $indx <= $#{ $mountlist }; $indx++ ) {
        my $spindle    = $mountlist->[ $indx ]->{ 'physical' };
        my $fstype     = $mountlist->[ $indx ]->{ 'fstype' };
        my $mountpoint = $mountlist->[ $indx ]->{ 'Mnt' };
        my $FmtCmd     = $mountlist->[ $indx ]->{ 'FmtCmd' };
        my $MntOpts    = $mountlist->[ $indx ]->{ 'MntOpts' };
        my @mkfs_cmd;
        my @mnt_cmd;

        # Fdisk the partition
        my $rescode = clean_spindle( $spindle );
        unless ( $rescode == 0 ) { return -1; }

        my $ddf = "/dev/" . $spindle . $ONE;

        # Make the mount point directory

        if ( !-d $mountpoint ) {
            if ( $DEBUG ) { print "Creating mountpoint $mountpoint\n"; }
            `mkdir $mountpoint`;
        } else {
            print "Mount point $mountpoint is okay\n";
        }

        if ( $fstype eq 'ext2' ) {
            if ( $FmtCmd eq 'NULL' ) {
                push( @mkfs_cmd, "mke2fs", "-F", $ddf );
            } else {
                push( @mkfs_cmd, $FmtCmd, '-t', $fstype, $ddf );
            }
            if ( $MntOpts eq 'NULL' ) {
                push( @mnt_cmd, 'mount', '-t', 'ext2', $ddf, $mountpoint );
            } else {
                push( @mnt_cmd,
                      'mount', ( split / /, $MntOpts ),
                      $ddf, $mountpoint );
            }

        } elsif ( $fstype eq 'ext3' ) {
            if ( $FmtCmd eq 'NULL' ) {
                push( @mkfs_cmd, "mkfs", "-t", "ext3", $ddf );
            } else {
                push( @mkfs_cmd, $FmtCmd, '-t', $fstype, $ddf );
            }
            if ( $MntOpts eq 'NULL' ) {
                push( @mnt_cmd,
                      'mount', '-t', 'ext3', '-o', 'data=writeback', $ddf,
                      $mountpoint );
            } else {
                push( @mnt_cmd,
                      'mount', ( split / /, $MntOpts ),
                      $ddf, $mountpoint );
            }

        } elsif ( $fstype eq 'jfs' ) {
            if ( $FmtCmd eq 'NULL' ) {
                push( @mkfs_cmd, "mkfs.jfs", "-q", $ddf );
            } else {
                push( @mkfs_cmd, $FmtCmd, '-t', $fstype, $ddf );
            }
            if ( $MntOpts eq 'NULL' ) {
                push( @mnt_cmd, 'mount', '-t', 'jfs', $ddf, $mountpoint );
            } else {
                push( @mnt_cmd,
                      'mount', ( split / /, $MntOpts ),
                      $ddf, $mountpoint );
            }

        } elsif ( $fstype eq 'reiserfs' ) {
            if ( $FmtCmd eq 'NULL' ) {
                push( @mkfs_cmd, "mkreiserfs", "--format", "3.6", "-f", $ddf );
            } else {
                push( @mkfs_cmd, $FmtCmd, '-t', $fstype, $ddf );
            }
            if ( $MntOpts eq 'NULL' ) {
                push( @mnt_cmd,
                      'mount',  '-t', $fstype, '-o',
                      'notail', $ddf, $mountpoint );
            } else {
                push( @mnt_cmd,
                      'mount', ( split / /, $MntOpts ),
                      $ddf, $mountpoint );
            }

        } elsif ( $fstype eq 'xfs' ) {
            if ( $FmtCmd eq 'NULL' ) {
                push( @mkfs_cmd, "mkfs.xfs", "-f", $ddf );
            } else {
                push( @mkfs_cmd, $FmtCmd, '-t', $fstype, $ddf );
            }
            if ( $MntOpts eq 'NULL' ) {
                push( @mnt_cmd, 'mount', '-t', 'xfs', $ddf, $mountpoint );
            } else {
                push( @mnt_cmd,
                      'mount', ( split / /, $MntOpts ),
                      $ddf, $mountpoint );
            }

        } elsif ( $fstype eq 'raw' ) {
            if ( $FmtCmd eq 'NULL' ) {
                push( @mkfs_cmd, "raw", $mountpoint, $ddf );
            } else {
                push( @mkfs_cmd, $FmtCmd, $mountpoint, $ddf );
            }
            if ( $MntOpts eq 'NULL' ) {
                push( @mnt_cmd, "echo", "no", "mount", "needed" );
            } else {
                push( @mnt_cmd, "echo", "no", "mount", "needed" );
            }

        } else {
            print STDERR "Bad fstype\n";
            return -1;
        }

        if ( $DEBUG ) { print "@mkfs_cmd\n"; }
        print "Making $fstype on $spindle\n";
        $rescode = system @mkfs_cmd;
        if ( $rescode != 0 ) {
            clean_spindle( $spindle );
            $rescode = system @mkfs_cmd;
            if ( $rescode != 0 ) {
                $realres = $rescode / 256;
                print STDERR "Failed mkfs for filesystem $fstype on $spindle\n";
                if ( $DEBUG ) { print "Result code: $rescode $realres\n"; }
                if ( $DEBUG ) { print "@mnt_cmd\n"; }
                return -1;
            }
        }
        print "Mounting $fstype on $mountpoint\n";
        $rescode = system @mnt_cmd;
        if ( $rescode != 0 ) {
            $realres = $rescode / 256;
            if ( $DEBUG ) { print "Result code: $rescode $realres\n"; }
            print STDERR "Failed mount on $spindle\n";
            return -1;
        }
    }
    return 0;
}

sub create_disk_layout {
    my ( $mntfile ) = @_;

    my $mntinfo = get_mountinfo( $mntfile );

    unless ( defined( $mntinfo ) ) {
        print STDERR "ERROR get_mountinfo failed \n";
        return -1;
    }

    my $res = setup_tools( $mntinfo );
    unless ( $res == 0 ) {
        print STDERR "ERROR: setup_tools failed\n";
        return -1;
    }
    my $disklist = assign_mountinfo( $mntinfo, $mntfile );
    if ( $disklist == -1 ) {
        print STDERR "Failed to assign mount info\n";
        return -1;
    }
    $res = make_fs( $disklist );
    return $res;
}

=head1 FUNCTION
make_one_disk

This functionw will be used by our actual command, for shell writers. We assume
the shell caller will default the last two options, or use NULL.

=cut

sub make_one_disk {
    my ( $fs_type, $min_size, $mount_point, $mFmtCmd, $mMntOpts ) = @_;
    my $mntinfo;
    my $mnt_item;

    $mnt_item->{ 'name' }   = "ScriptRequest";
    $mnt_item->{ 'fstype' } = $fs_type;
    $mnt_item->{ 'size' }   = $min_size;
    $mnt_item->{ 'Mnt' }    = $mount_point;
    if ( ( !$mFmtCmd ) || ( $mFmtCmd eq " " ) ) {
        $mnt_item->{ "FmtCmd" } = "NULL";
    } else {
        $mnt_item->{ "FmtCmd" } = $mFmtCmd;
    }
    if ( ( !$mMntOpts ) || ( $mMntOpts eq " " ) ) {
        $mnt_item->{ "MntOpts" } = "NULL";
    } else {
        $mnt_item->{ "MntOpts" } = $mMntOpts;
    }

    push( @$mntinfo, $mnt_item );
    my $res = setup_tools( $mntinfo );
    unless ( $res == 0 ) {
        print STDERR "ERROR: setup_tools failed\n";
        return -1;
    }
    my $disklist = assign_single( $mntinfo );
    if ( $disklist == -1 ) {
        print STDERR "Failed to assign mount info\n";
        return -1;
    }
    $res = make_fs( $disklist );
    return $res;

}

1;

