#!/usr/bin/perl -w

# Cliffw, OSDL 2004
# A helper package for stp clients

package STP::Client::Tools;

use strict;
use English;
use FileHandle;
use vars qw(@ISA @EXPORT);
use Exporter;
use Cwd;
my $DEBUG = 0;

@ISA = qw(Exporter);

@EXPORT = qw(
  console_note
  syslog
  os_detect
  get_bits
  install_tar
  install_rpm
);

sub syslog;
=head1 NAME 
Tools

=head1 SYNOPSIS
A central place to put various useful functions necessary to test automation.
Can be used by test authors, is used by the STP engine. 

=cut

=head1 FUNCTION
console_note( message )

Sends a timestamped message to /dev/console, for the benefit of
any humans around. Should be used frequently by test authors.
We'd like to know what the test was doing before it died.

=cut

#
# console_note() produce output for ops
sub console_note {
    my ( $msg ) = @_;
    my $time = `date +'%D %T'`;

    chomp $time;

    if ( open( F, ">>/dev/console" ) ) {
        print F "$time $msg\n";
        close F;
    } else {
        syslog("Cannot send console message");
    }
}

=head1 FUNCTION
syslog( message )

Places a message in the system log. The system log will be copied to
the test results directory
=cut

#
# syslog() messages will be visible to the end user in syslog
#
sub syslog {
    system "logger -s -t STP -- \"@_\"";
}

=head1 FUNCTION os_detect()

Checks the /etc/*release file to identify the distro used
The magic file names are at the start of the function
    # SuSE: /etc/SuSE-release
    # Redhat: /etc/redhat-release
    # Debian: /etc/debain_version
Extracts version information from RedHat and SuSE 
Returns a string

=cut

sub os_detect {

    my %os_list = (
                    SuSE   => "/etc/SuSE-release",
                    RH     => "/etc/redhat-release",
                    Debian => "/etc/debian_version",
    );
    my $os_string;
    my $os_ver;
    my $os_raw;

    if ( -f $os_list{ 'SuSE' } ) {
        $os_string = "SuSE";

        # not yet needed
        $os_raw = `egrep "^VERSION" $os_list{'SuSE'}`;
        ( $os_ver = $os_raw ) =~ s/VERSION = //;

        # When we need version info, uncomment next line
        # $os_string = $os_string . '_' . $os_ver;
    } elsif ( -f $os_list{ 'RH' } ) {
        $os_string = "RH";
        $os_raw    = `grep "Red" $os_list{ 'RH' }`;
        chomp $os_raw;
        $os_ver = ( $os_raw =~ m/(\d+\.?\d*|\.\d+)/g )[ 0 ];
        $os_string = $os_string . "_" . $os_ver;
    } elsif ( -f $os_list{ 'Debian' } ) {
        $os_string = "Debian";
    } else {
        $os_string = "NOT_FOUND";
    }
    return $os_string;
}

=head1 FUNCTION 
get_bits( file_name, url_for_wget )

Given a filename and a wget host, first check to see if the file 
exists, otherwise go and get it. 

=cut

sub get_bits {
    my ( $filename, $wget_host ) = @_;
    my $pwd    = cwd();
    my $search = $pwd . "/" . $filename;

    unless ( ( $filename ) && ( $wget_host ) ) {
        print STDERR "get_bits - missing parameters\n";
        return -1;
    }

    if ( $DEBUG )     { print "Searching for $search\n"; }
    if ( -f $search ) { return 0; }
    if ( $DEBUG )     { print "Trying wget for $filename \n"; }

    $search = $wget_host . "/" . $filename;

    system( "wget -q $search" ) && return -1;

    # Otherwise, return okay
    $search = "";
    return 0;
}

=head1 FUNCTION
install_tar( source_dir_name, archive_file_name, list_of_build_commands )

This is a start on a generic installation function. It assumes a gzipped tarball as
the source. The final paramter is an array of string which are the actual commands
used to configure, make and install the program
It requires the caller to give both the source file name, and the directory name
after the unpacking. I suppose this parsing would be better done inside this
function, so send in a patch :)

TODO:
Add a method of dealing with interactive configure routines
=cut

# A generic routine to untar, compile and install
# gzipped tarballs - tarball should be named the same
# as the directory
sub install_tar {
    my ( $dirname, $tarball, @build_commands ) = @_;
    my $command;
    my $pwd = cwd();

    # cleanup
    if ( -f $dirname ) { `rm -rf $dirname`; }

    if ( ( $tarball =~ /.*.tar.gz$/ ) || ( $tarball =~ /.*.tgz/ ) ) {
        system( "tar xzf ./$tarball" ) && return -1;
    } elsif ( $tarball =~ /.*.tar.bz2/ ) {
        system( "tar xjf ./$tarball" ) && return -1;
    } else {
        print STDERR "Can't install- unknown archive type \n";
        return -1;
    }

    chdir "$dirname" || return -1;
    foreach $command ( @build_commands ) {

        # my $LOG =
          # File::Temp( "install_gz.log-XXXXXX", DIR => '/tmp' )->new->filename;
	# doesn't work on STP, sorry
	my $LOG = "/tmp/" . $dirname . ".log";
        system( "$command >> $LOG 2>&1" ) && return -1;
    }

    chdir $pwd;

    # Otherwise, we assume life is good
    return 0;
}

=head1 FUNCTION
install_rpm( name_of_rpm, url_for_wget )

Given an rpm, first checks if the archive is in the current directory
If not found, is obtained via wget, then installed

TODO - check for existence of previous version, perhaps force update
TODO - add install_deb

=cut

# Find and install in one pass
sub install_rpm {
    my ( $rpmname, $wget_host ) = @_;
    my $pwd    = cwd();
    my $search = $pwd . "/" . $rpmname;

    if ( -f $search ) { return 0; }
    $search = $wget_host . "/" . $rpmname;
    system( "wget -q $search" ) && return -1;

    system( "LD_ASSUME_KERNEL=2.2.5 rpm -ivh ./$rpmname" ) && return -1;

    return 0;
}

