# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Database Package
#
# Author:  Jerry Sievert
# Date: 10/29/01
#
# Database module, given a config file, handle all interface with the
# database, as well as track/store everything.

=head1 NAME STP::DB::Database

DBI wrapper for STP.

 uses -> STP::DB::ResultSet

 defines -> $dbh, $dbh_lock, $DBG

=cut


package STP::DB::Database;

use strict;
use warnings;
use DBI;
use STP::DB::ResultSet;

BEGIN { }

# variables that are used throughout the children classes
my $dbh;         # Database Handle
my $dbh_lock;    # database handle lock

my $DBG = 0;     # debug?

# base constructor




=head2 FUNCTION new
 
constructor, initalizes the usual anon hash
 this should be what everybody else calls via SUPER, but nooooo...
 always connects to the database via _connect
 Apparently implents some simple connection locking, but this
 looks a bit more like a ref count - connection is removed when
 dbh_lock == 0

 TODO
 proper bless 
 check input params?
 Document other functions that touch dbh_lock

=cut 

sub new {
    # create an empty hash, and "bless" it
    my $self = {};
    bless $self;

    shift;

    # we're passed a config at instanciation
    ( $self->{ config } ) = @_;

    $self->{ resultSet } = {};

    # do some sanity settings, set everything to nothing
    $self::dbh      = "";
    $self::dbh_lock = 0;

    $self->_connect();

    return $self;
}

# connect to the database, increment the number of locks

=head1 FUNCTION _connect 

Always increments dbh_lock
 If dbh_lock == 1 we connect to the database using 
connection info from the config object.
 Otherwise, call _warn function
 If connection fails, call the _warn function
 No return 

=cut

sub _connect {
    my ( $self ) = @_;

    $self::dbh_lock++;

    if ( $self::dbh_lock == 1 ) {
        $self->_warn( "Connecting to the database" );

        $self::dbh =
          DBI->connect( $self->{ config }->getElementValue( 'database.dsn' ),
                                   $self->{ config }->getElementValue(
                                                          'database.username' ),
                                   $self->{ config }->getElementValue(
                                                        'database.password' ) );

        if ( defined( $DBI::err ) || defined( $DBI::errstr ) ) {
            $self->_warn( "Database connect returned error: $DBI::err" );
            $self->_warn( "Translation: $DBI::errstr" );
        }
    } else {
        $self->_warn( "Increasing count for \$dbh_lock to $self::dbh_lock" );
    }
}

=head1 FUNCTION _disconnect

 Decrement dbh_lock and disconnect if == 0
 No return

=cut

# disconnect from the database (or decrement the number of locks)
sub _disconnect {
    my ( $self ) = @_;

    $self::dbh_lock--;

    if ( $self::dbh_lock == 0 ) {
        $self->_warn( "Disconnecting from the database" );
        $self::dbh->disconnect();
    } else {
        $self->_warn( "Decreasing count for \$dbh_lock to $self::dbh_lock" );
    }
}

=head1 FUNCTION _warn

Prints msg if DBG

=cut

# internal debugging warnings
sub _warn {
    my ( $self, $msg ) = @_;

    if ( $self::DBG ) {
        print STDERR $msg . "\n";
    }
}

=head1 FUNCTION _update

 Calls _connect()
 Uses DBI::do() to execute query.
 No error checking
 Calls _disconnect()
 No return

=cut 

# do an update (create, insert, update, delete)
sub _update {
    my ( $self, $query ) = @_;

    $self->_connect;

    my $result;

    $result = $self::dbh->do( $query );

    $self->_disconnect;

    return $result;
}

# do a select
=head1 FUNCTION _select

 Input: query
 Calls _connect
 Uses DBI::prepare and dies on failure
 Returns either a filled out result set or a new empty result set (??)
 Calls _disconnect
 Adds copy of result set to object

=cut 

sub _select {
    my ( $self, $query ) = @_;

    $self->_connect;

    my $sth = $self::dbh->prepare( $query );

    die "invalid query: $query" unless $sth;

    my $rs;
    if ( $sth->execute() ) {
        $rs = new STP::DB::ResultSet( $sth );
    } else {
        $rs = new STP::DB::ResultSet();
    }

    $sth->finish();

    $self->_disconnect;

    # save a local copy
    $self->{ resultSet } = $rs;

    return $rs;
}

# return the resultset that we're using to the caller
sub getResultSet {
    my ( $self ) = @_;

    return $self->{ resultSet };
}

=head1 Accessor functions

 getResultSet
 getProperty
 getNumRows

=cut

# get property, it really checks the hash returned from the query in resultset
sub getProperty {
    my ( $self, $row, $property ) = @_;
    my $property_value;

    eval { $property_value = $self->{ resultSet }->getProperty( $row, $property ); };
    if ( $@ ) {
        return "";
    }

    return $property_value;
}

# return the number of rows of data available
sub getNumRows {
    my ( $self ) = @_;

    return $self->{ resultSet }->getNumRows;
}

END { }

1;

