# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::DB::Host;

@ISA = qw( STP::DB::Database );

use strict;
use warnings;

use STP::DB::Database;
use STP::DB::HostState;
use STP::DB::StateTimer;

BEGIN { }

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " host ";

    $self->{ fields } =
      "h.uid, h.descriptor, h.description, h.ram, "
      . "h.disk, h.host_state_uid, h.host_type_uid";

    return $self;
}

# create the table (only used at setup time)
sub createTable {
    my ( $self ) = @_;

    # this is gross and ugly, especially setting database specific types
    # do not try this at home, and this should eventually be redone to provide
    # for more generic database support (perhaps with a switch for the db type)
    my $query = qq(
		    CREATE TABLE $self->{ tableName }
		    ( uid INT, descriptor VARCHAR(20),
		      description TEXT, ram DOUBLE, disk INT,
		      host_state_uid INT, host_type_uid INT
		    )
		   );

    $self->_update( $query );
}

sub getHostByUid {
    my ( $self, $uid ) = @_;

    my $query = qq(
		   SELECT $self->{ fields }
		   FROM $self->{ tableName } h
		   WHERE uid = $uid
		   );

    $self->{ resultSet } = $self->_select( $query );
}

sub getHostByDescriptor {
    my ( $self, $descriptor ) = @_;

    # Trim off the domain if it's present
    if ( $descriptor =~ /^(.+?)\./ ) {
        $descriptor = $1;
    }

    my $query =
qq(SELECT $self->{ fields } FROM $self->{ tableName } h WHERE h.descriptor = '$descriptor');

    $self->{ resultSet } = $self->_select( $query );

    return $self->getProperty( 0, "uid" );
}

sub writeHost {
    my (
         $self, $uid,  $descriptor,     $description,
         $ram,  $disk, $host_state_uid, $host_type_uid
      )
      = @_;

    my $query;

    # If we are handed a code for the uid, resolve it first
    if ( $host_state_uid =~ /^\d+$/ ) {
        my $hstate = new STP::DB::HostState( $self->{ config } );

        $host_state_uid = $hstate->getHostStateByCode( $host_state_uid );
    }

    # do we insert or update?
    if ( $uid == 0 ) {

        # we're inserting, so get a new uid
        $query = qq(
		     INSERT INTO $self->{ tableName }
		     ( descriptor, description, ram,
		       disk, host_state_uid, host_type_uid )
		     VALUES
		     (
		       '$descriptor', '$description',
		       $ram, $disk, $host_state_uid, $host_type_uid )
		    );
    }

    # update
    else {
        $query = qq(
		    UPDATE $self->{ tableName }
		    SET descriptor = '$descriptor',
		    description = '$description',
		    ram = $ram, disk = $disk, host_state_uid = $host_state_uid,
		    host_type_uid = $host_type_uid
		    WHERE uid = $uid
		   );
    }

    $self->_update( $query );

    return $uid;
}

sub getFreeHostsByHosttype {
    my ( $self, $hosttype ) = @_;

    my $hstate = new STP::DB::HostState( $self->{ config } );
    my $avail  = $hstate->getHostStateByCode( "avail" );

    my $query = qq(
		   SELECT $self->{ fields }
		   FROM $self->{ tableName } h
		   WHERE h.host_state_uid = $avail AND h.host_type_uid = $hosttype
		   ORDER BY h.uid
		   );

    $self->{ resultSet } = $self->_select( $query );
}

sub setHostState {
    my ( $self, $code ) = @_;

    my $uid = $self->getProperty( 0, "uid" );

    my $hstate = new STP::DB::HostState( $self->{ config } );
    my $state  = $hstate->getHostStateByCode( $code );

    my $query = "UPDATE host SET host_state_uid = $state WHERE uid = $uid";
    $self->_update( $query );

    # clear the state timer
    my $st = new STP::DB::StateTimer( $self->{ config } );
    $st->setStateTimer( $uid, $state, 0 );
}

sub getHostState {
    my ( $self, $uid ) = @_;

    if ( $uid ) {
        $self->getHostByUid( $uid );
    } else {
        $uid = $self->getProperty( 0, "uid" );
    }

    return "MISSING UID" unless $uid;

    my $hstate = new STP::DB::HostState( $self->{ config } );
    my $state  =
      $hstate->getHostStateByUid( $self->getProperty( 0, "host_state_uid" ) );

    return $state;
}

# given a test request uid, return the host assigned to it
# this will need to be an array for multi-host tests
sub getHostForTestRequest {
    my ( $self, $test_uid ) = @_;

    my $query =
qq( SELECT host_uid FROM test_request_to_host WHERE test_request_uid = $test_uid );

    $self->_select( $query );

    if ( $self->getNumRows() == 0 ) {
        return undef;
    }

    my $huid = $self->getProperty( 0, "host_uid" );

    $self->getHostByUid( $huid );

    return $huid;
}

END { }

1;
