# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::DB::HostType;
@ISA = qw( STP::DB::Database );

use strict;
use STP::XML::HostType;
use STP::DB::Database;

BEGIN { }

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " host_type ";

    $self->{ fields } =
      "ht.uid, ht.descriptor, ht.cpu, ht.ram_qty, "
      . " ht.storage_space, ht.spindle_qty, ht.eth100, "
      . " ht.eth1000";

    return $self;
}

# create the table (only used at setup time)
sub createTable {
    my ( $self ) = @_;

    my $query = qq(
		    CREATE TABLE $self->{ tableName }
		    (
		      uid INT, descriptor VARCHAR(20), cpu VARCHAR(255),
		      ram_qty INT, storage_space INT, spindle_qty INT,
		      eth100 VARCHAR(20), eth1000 VARCHAR(20)
		    )
		   );

    $self->_update( $query );
}

sub getHostTypeByUid {
    my ( $self, $uid ) = @_;

    my $query = qq(
		   SELECT $self->{ fields } 
		   FROM $self->{ tableName } ht
		   WHERE ht.uid = $uid
		   );

    $self->{ resultSet } = $self->_select( $query );
}

# load all of the hosts into an array, return the array
sub getAllHostTypes {
    my ( $self ) = @_;

    # get all of the uid's
    my $query = qq(
		    SELECT uid
		    FROM $self->{ tableName }
		    ORDER BY uid
		   );

    $self->{ resultSet } = $self->_select( $query );

    return $self->arrayOfResults();
}

# load all of the hosts into an array, return the array
sub getAllHostTypesForTest {
    my ( $self, $test_uid ) = @_;

    # get all of the uid's
    my $query = qq(
		    SELECT ht.uid as uid
		    FROM $self->{ tableName } ht, test_host_type tht
		    WHERE tht.host_type_uid = ht.uid AND tht.test_uid = $test_uid
		    ORDER BY ht.uid
		   );

    $self->{ resultSet } = $self->_select( $query );

    return $self->arrayOfResults();
}

# given a resultset of uid's, create an array of HostType objects,
# fill it up, and return it.
sub arrayOfResults() {
    my ( $self ) = @_;

    my $count = $self->getNumRows();

    # if there are no results, return an undef
    if ( $count == 0 ) {
        return undef;
    }

    my @data = ();

    for ( my $i = 0; $i < $count; $i++ ) {
        my $uid = $self->getProperty( $i, "uid" );

        # pass our config along
        my $ht = new STP::DB::HostType( $self->{ config } );

        $ht->getHostTypeByUid( $uid );
        $data[ $i ] = $ht;
    }

    return @data;
}

# save a host to the database
sub writeHostType {
    my (
         $self,        $uid,     $descriptor,
         $cpu,         $ram_qty, $storage_space,
         $spindle_qty, $eth100,  $eth1000
      )
      = @_;

    my $query;

    # do we insert or update?
    if ( $uid == 0 ) {

        # we're inserting, so get a new uid
        $query = qq(
		    INSERT INTO $self->{ tableName }
		    ( descriptor, cpu, ram_qty, storage_space,
		      spindle_qty, eth100, eth1000 )
		    VALUES
		    ( '$descriptor', '$cpu', $ram_qty,
		      $storage_space, $spindle_qty, '$eth100', '$eth1000'
		    )
		   );
    }

    # update
    else {
        $query = qq(
		    UPDATE $self->{ tableName }
		    SET descriptor = '$descriptor', cpu = '$cpu',
		    ram_qty = $ram_qty, storage_space = $storage_space,
		    spindle_qty = $spindle_qty, eth100 = $eth100,
		    eth1000 = $eth1000
		    WHERE uid = $uid
		   );
    }

    $self->_update( $query );

    return $uid;
}

# convert to an xml document
sub toXML {
    my ( $self, $number ) = @_;

    my $xml = new STP::XML::HostType();

    $xml->setElementValue( "uid", $self->getProperty( $number, "uid" ) );
    $xml->setElementValue( "descriptor",
                           $self->getProperty( $number, "descriptor" ) );
    $xml->setElementValue( "cpu", $self->getProperty( $number, "cpu" ) );
    $xml->setElementValue( "ram_qty",
                           $self->getProperty( $number, "ram_qty" ) );
    $xml->setElementValue( "storage_space",
                           $self->getProperty( $number, "storage_space" ) );
    $xml->setElementValue( "spindle_qty",
                           $self->getProperty( $number, "spindle_qty" ) );
    $xml->setElementValue( "eth100", $self->getProperty( $number, "eth100" ) );
    $xml->setElementValue( "eth1000",
                           $self->getProperty( $number, "eth1000" ) );

    return $xml;
}

sub toHostInfoXML {
    my ( $self, $number ) = @_;

    my $xml = new STP::XML::HostInfo();
}

END { }

1;
