# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Filename: QDXml.pm
# Author: Jerry Sievert (jerry@osdl.org)
# Date: Feb 18, 2002
#
# Quick/Dirty XML
#
# This is the base container that holds XML Elements.
#
# It's defined this way because it I wanted to make sure that
# the API matched the existing QDXml code package written in
# Java.  Having the same API under multiple languages is a
# major timesaver for me.

package STP::XML::QDXml;

@ISA = qw( STP::XML::QDObject );

use strict;
use warnings;
use STP::XML::QDObject;
use STP::XML::QDXmlElement;

BEGIN { }

# instanciate a new QDXml
# possible arguments are as follows:
#
#   elementName (container name)
sub new {

    # instanciate the data, fill in values
    my $self = {};
    bless $self;

    shift;

    my ( $elementName ) = @_;

    if ( defined( $elementName ) ) {
        $self->{ elementName } = $elementName;
    } else {

        # should be defined, even if empty
        $self->{ elementName } = "";
    }

    # create an array to hold the data

    $self->{ data } = [];
    return $self;
}

# return the data type of the object - in this case, a RECORD
sub getDataType {
    my ( $self ) = @_;
    return 1;
}

# returns the name of the element being referenced, or if the case
# of a full document, the object
sub getElementName {
    my ( $self ) = @_;
    return $self->{ elementName };
}

# sets the elementName, or in this case, the container name
sub setElementName {
    my ( $self, $elementName ) = @_;
    $self->{ elementName } = $elementName;
}

# adds an element to the container
# possible arguments are as follows:
#
#   elementName
#   elementValue
#
# or:
#
#   QDXmlObject (either QDXml or QDXmlElement)
sub addElement {
    my ( $self ) = @_;
    shift;

    # how many arguments do we have?
    # if we have 1, it should be a QDXmlElement
    # if we have 2, it should be a name/value pair
    if ( @_ == 1 ) {
        my ( $element ) = @_;

        my $data = $self->{ data };

        # probably not the most efficient way in perl to do this, but ...
        my $count = scalar( @$data );
        @$data[ $count ] = $element;
    } elsif ( @_ == 2 ) {
        my ( $elementName, $elementValue ) = @_;

        my $data = $self->{ data };

        my $element = new STP::XML::QDXmlElement( $elementName, $elementValue );

        my $count = scalar( @$data );
        @$data[ $count ] = $element;
    }
}

# sets the value of an element contained
# arguments:
#
# elementName
# elementValue
sub setElementValue {
    my ( $self, $elementName, $elementValue ) = @_;

    my $sub = 0;
    my $topField;
    my $bottomField;

    # first, check to see if we're a sub-record
    if ( $elementName =~ /\./ ) {
        ( $topField, $bottomField ) = split ( /\s*\.\s*/, $elementName, 2 );
        $sub = 1;
    }

    my $elements = $self->{ data };
    my $count    = scalar( @$elements );

    for ( my $i = 0; $i < $count; $i++ ) {
        my $element = @$elements[ $i ];

        # are we a record?
        if ( $element->getDataType() eq 1 ) {

            # are we a sub?
            if ( $sub eq 1 ) {
                if ( $element->getElementName() eq $topField ) {
                    $element->setElementValue( $bottomField, $elementValue );
                    return;
                }
            }
        }

        if ( $element->getElementName() eq $elementName ) {
            $element->setElementValue( $elementValue );
            return;
        }
    }
}

# gets an elementValue contained in the object
sub getElementValue {
    my ( $self, $elementName ) = @_;

    my $sub = 0;
    my $topField;
    my $bottomField;

    # first, check to see if we're a sub-record
    if ( $elementName =~ /\./ ) {
        ( $topField, $bottomField ) = split ( /\s*\.\s*/, $elementName, 2 );
        $sub = 1;
    }

    my $elements = $self->{ data };
    my $count    = scalar( @$elements );

    for ( my $i = 0; $i < $count; $i++ ) {
        my $element = @$elements[ $i ];

        # are we a record?
        if ( $element->getDataType() eq 1 ) {

            # are we a sub?
            if ( $sub eq 1 ) {
                if ( $element->getElementName() eq $topField ) {
                    return $element->getElementValue( $bottomField );
                }
            }
        }

        if ( $element->getElementName() eq $elementName ) {
            return $element->getElementValue();
        }
    }

    return "";
}

# returns the string representation of the object
sub toString {
    my ( $self, $depth ) = @_;

    # if we don't have an elementName, we just return nothing
    if ( $self->{ elementName } eq "" ) {
        return "";
    }

    if ( !defined( $depth ) ) {
        $depth = 0;
    }

    my $data = "";

    # set our depth
    if ( defined( $depth ) && $depth > 0 ) {
        $data .= " " x $depth;
    }

    # add the elementName
    $data .= "<" . $self->{ elementName } . ">\n";

    # iterate through the elements, relying on their toString() methods
    my $elements = $self->{ data };
    my $count    = scalar( @$elements );

    for ( my $i = 0; $i < $count; $i++ ) {
        my $element = @$elements[ $i ];
        $data .= $element->toString( $depth + 1 );
    }

    # set our depth
    if ( defined( $depth ) && $depth > 0 ) {
        $data .= " " x $depth;
    }

    # add the closing elementName
    $data .= "</" . $self->{ elementName } . ">\n";

    return $data;
}

# parse an xml string, load its data into the container
sub parseXMLData {
    my ( $self, $xmlData ) = @_;

    # if we have no elementName, simply return, doing nothing
    if ( $self->{ elementName } eq "" ) {
        return;
    }

    # strip ourselves from the record
    $xmlData = getBareElement( $self->{ elementName }, $xmlData );

    if ( $xmlData eq "" ) {
        return;
    }

    # iterate through all of the entries and force them to parse for
    # their children
    my $elements = $self->{ data };
    my $count    = scalar( @$elements );
    for ( my $i = 0; $i < $count; $i++ ) {
        my $element = @$elements[ $i ];
        $element->parseXMLData( $xmlData );
    }

}

sub getBareElement {
    my ( $elementName, $xmlData ) = @_;

    if ( $xmlData =~ /<$elementName>(.*?)<\/$elementName>/si ) {
        return $1;
    }

    return "";
}

END { }

1;
